@@ -1,7 +1,8 @@
 from .analysis import Analysis
+from .child import Child
 from .datafile import Datafile
 from .dataset import Dataset
 from .manifest import Manifest
 
 
-__all__ = "Analysis", "Datafile", "Dataset", "Manifest"
+__all__ = "Analysis", "Child", "Datafile", "Dataset", "Manifest"
