@@ -228,17 +228,17 @@ def unwrap(fcn):
 
 
 class AppFrom:
-    """Context manager that imports module 'app' from user's code base at a location app_path.
-
-     The manager will issue a warning if an existing module called "app" is already loaded.
-
-     The manager makes a temporary addition to the system path (to ensure app is loaded from the correct path)
-
-     The manager will unload the module (by deleting it from sys.modules) on exit, enabling
+    """A context manager that imports the module "app" from a file named "app.py" in the given directory on entry (by
+    making a temporary addition to the system path) and unloads it (by deleting it from `sys.modules`) on exit. It will
+    issue a warning if an existing module called "app" is already loaded. Usage example:
 
+    ```python3
     with AppFrom('/path/to/dir') as app:
         Runner().run(app)
+    ```
 
+    :param str app_path: path to directory containing module named "app.py".
+    :return None:
     """
 
     def __init__(self, app_path="."):
@@ -250,26 +250,37 @@ class AppFrom:
         # Warn on an app present on the system path
         if "app" in sys.modules.keys():
             logger.warning(
-                "Module 'app' already on system path. Using 'AppFrom' context will yield unexpected results. Avoid using 'app' as a python module, except for your main entrypoint"
+                "Module 'app' already on system path. Using 'AppFrom' context will yield unexpected results. Avoid "
+                "using 'app' as a python module, except for your main entrypoint."
             )
 
-        # Insert the present directory first on the system path
+        # Insert the present directory first on the system path.
         sys.path.insert(0, self.app_path)
 
-        # Import the app from the present directory
+        # Import the app from the present directory.
         self.app_module = importlib.import_module("app")
 
         # Immediately clean up the entry to the system path (don't use "remove" because if the user has it in their
         # path, this'll be an unexpected side effect, and don't do it in cleanup in case the called code inserts a path)
         sys.path.pop(0)
         logger.debug("Imported app at app_path and cleaned up temporary modification to sys.path %s", self.app_path)
-
         return self
 
     def __exit__(self, exc_type, exc_value, traceback):
-        # Unload the imported module
-        del sys.modules["app"]
-        logger.debug("Deleted app from sys.modules and cleaned up (app_path %s)", self.app_path)
+        """Unload the imported module.
+
+        :return None:
+        """
+        try:
+            del sys.modules["app"]
+            logger.debug("Deleted app from sys.modules")
+
+        except KeyError:
+            context_manager_name = type(self).__name__
+            logger.warning(
+                f"Module 'app' was already removed from the system path prior to exiting the {context_manager_name} "
+                f"context manager. Using the {context_manager_name} context may yield unexpected results."
+            )
 
     @property
     def run(self):
