@@ -9,6 +9,7 @@ from unittest import TestCase, mock
 import twined.exceptions
 from octue.cloud.deployment.google import cloud_run
 from octue.cloud.pub_sub.service import Service
+from octue.exceptions import MissingServiceID
 from octue.resources.service_backends import GCPPubSubBackend
 from tests.cloud.pub_sub.mocks import MockTopic
 
@@ -23,16 +24,16 @@ class TestCloudRun(TestCase):
     def test_post_to_index_with_no_payload_results_in_400_error(self):
         """Test that a 400 (bad request) error code is returned if no payload is sent to the Flask endpoint."""
         with cloud_run.app.test_client() as client:
-            response = client.post("/")
+            response = client.post("/", json={"deliveryAttempt": 1})
             self.assertEqual(response.status_code, 400)
 
     def test_post_to_index_with_invalid_payload_results_in_400_error(self):
         """Test that a 400 (bad request) error code is returned if an invalid payload is sent to the Flask endpoint."""
         with cloud_run.app.test_client() as client:
-            response = client.post("/", json={"some": "data"})
+            response = client.post("/", json={"some": "data", "deliveryAttempt": 1})
             self.assertEqual(response.status_code, 400)
 
-            response = client.post("/", json={"message": "data"})
+            response = client.post("/", json={"message": "data", "deliveryAttempt": 1})
             self.assertEqual(response.status_code, 400)
 
     def test_post_to_index_with_valid_payload(self):
@@ -43,6 +44,7 @@ class TestCloudRun(TestCase):
                 response = client.post(
                     "/",
                     json={
+                        "deliveryAttempt": 1,
                         "subscription": "projects/my-project/subscriptions/my-subscription",
                         "message": {
                             "data": base64.b64encode(
@@ -55,6 +57,51 @@ class TestCloudRun(TestCase):
 
                 self.assertEqual(response.status_code, 204)
 
+    def test_error_raised_when_no_service_id_environment_variable(self):
+        """Test that a MissingServiceID error is raised if the SERVICE_ID environment variable is missing."""
+        with self.assertRaises(MissingServiceID):
+            cloud_run.answer_question(
+                project_name="a-project-name",
+                question={
+                    "data": {},
+                    "attributes": {"question_uuid": "8c859f87-b594-4297-883f-cd1c7718ef29"},
+                },
+                credentials_environment_variable="GOOGLE_APPLICATION_CREDENTIALS",
+            )
+
+    def test_error_raised_when_service_id_environment_variable_is_empty(self):
+        """Test that a MissingServiceID error is raised if the SERVICE_ID environment variable is empty."""
+        with mock.patch.dict(os.environ, {"SERVICE_ID": ""}):
+            with self.assertRaises(MissingServiceID):
+                cloud_run.answer_question(
+                    project_name="a-project-name",
+                    question={
+                        "data": {},
+                        "attributes": {"question_uuid": "8c859f87-b594-4297-883f-cd1c7718ef29"},
+                    },
+                    credentials_environment_variable="GOOGLE_APPLICATION_CREDENTIALS",
+                )
+
+    def test_redelivered_questions_are_acknowledged_and_ignored(self):
+        """Test that redelivered questions are acknowledged and then ignored."""
+        with cloud_run.app.test_client() as client:
+            with mock.patch("octue.cloud.deployment.google.cloud_run.answer_question") as mock_answer_question:
+
+                response = client.post(
+                    "/",
+                    json={
+                        "deliveryAttempt": 2,
+                        "subscription": "projects/my-project/subscriptions/my-subscription",
+                        "message": {
+                            "data": {},
+                            "attributes": {"question_uuid": str(uuid.uuid4()), "forward_logs": "1"},
+                        },
+                    },
+                )
+
+        self.assertEqual(response.status_code, 204)
+        mock_answer_question.assert_not_called()
+
     def test_with_no_deployment_configuration_file(self):
         """Test that the Cloud Run `answer_question` function uses the default deployment values when a deployment
         configuration file is not provided.
